<?php
/**
 * SphiderLite Version 2.6.*
 * Sphider is licensed under the GNU GPL.
 *
 * PHP version 5.4
 *
 * @category Sphider_Search
 * @package  Admin,Search
 * @author   R. Prew  <CaptQ@blog.worldspaceflight.com>
 * @license  GNU GPL
 * @link     http://www.sphidersearch.com
 *
 * Original base code by:
 *   Ando Saabas  ando(a t)set.ee
 */

$includes = array('./include', 'include', '../include');
require_once 'stem_class.php';
if (!in_array($include_dir, $includes) ) {
    die("Illegal include.");
}

$special_characters_array = array(
    '~',
    '!',
    '@',
    '#',
    '$',
    '%',
    '^',
    '&',
//    '*', // This breaks the wildcard search
    '(',
    ')',
    '_',
//    '-', // This breaks the negate search
    '+',
    '=',
    '{',
    '}',
    '[',
    ']',
    '|',
    '<',
    '>',
//    ',', // Decimal separator, dealt with separately
//    '.', // Decimal separator, dealt with separately
    ';',
    ':',
    '?',
    '"',
    '\/',
    '\\');

// All lower case intentional
$entities = array(
    "&nbsp;" => " ",
    "&amp;" => "&",
    "&apos;" => "'",
    "&THORN;"  => "Þ",
    "&szlig;"  => "ß",
    "&agrave;" => "à",
    "&aacute;" => "á",
    "&acirc;"  => "â",
    "&atilde;" => "ã",
    "&auml;"   => "ä",
    "&aring;"  => "å",
    "&aelig;"  => "æ",
    "&ccedil;" => "ç",
    "&egrave;" => "è",
    "&eacute;" => "é",
    "&ecirc;"  => "ê",
    "&euml;"   => "ë",
    "&igrave;" => "ì",
    "&iacute;" => "í",
    "&icirc;"  => "î",
    "&iuml;"   => "ï",
    "&eth;"    => "ð",
    "&ntilde;" => "ñ",
    "&ograve;" => "ò",
    "&oacute;" => "ó",
    "&ocirc;"  => "ô",
    "&otilde;" => "õ",
    "&ouml;"   => "ö",
    "&oslash;" => "ø",
    "&ugrave;" => "ù",
    "&uacute;" => "ú",
    "&ucirc;"  => "û",
    "&uuml;"   => "ü",
    "&yacute;" => "ý",
    "&thorn;"  => "þ",
    "&yuml;"   => "ÿ",
    "&Agrave;" => "à",
    "&Aacute;" => "á",
    "&Acirc;"  => "â",
    "&Atilde;" => "ã",
    "&Auml;"   => "ä",
    "&Aring;"  => "å",
    "&Aelig;"  => "æ",
    "&Ccedil;" => "ç",
    "&Egrave;" => "è",
    "&Eacute;" => "é",
    "&Ecirc;"  => "ê",
    "&Euml;"   => "ë",
    "&Igrave;" => "ì",
    "&Iacute;" => "í",
    "&Icirc;"  => "î",
    "&Iuml;"   => "ï",
    "&ETH;"    => "ð",
    "&Ntilde;" => "ñ",
    "&Ograve;" => "ò",
    "&Oacute;" => "ó",
    "&Ocirc;"  => "ô",
    "&Otilde;" => "õ",
    "&Ouml;"   => "ö",
    "&Oslash;" => "ø",
    "&Ugrave;" => "ù",
    "&Uacute;" => "ú",
    "&Ucirc;"  => "û",
    "&Uuml;"   => "ü",
    "&Yacute;" => "ý",
    "&Yhorn;"  => "þ",
    "&Yuml;"   => "ÿ",
    "&Cacute;" => "ć",
    "&cacute;" => "ć",
    "&OElig;"  => "œ",
    "&oelig;"  => "œ",
    "&Scaron;" => "š",
    "&scaron;" => "š",
    "&fnof;"   => "ƒ");

//Apache multi indexes parameters
$apache_indexes = array(
    "N=A" => 1,
    "N=D" => 1,
    "M=A" => 1,
    "M=D" => 1,
    "S=A" => 1,
    "S=D" => 1,
    "D=A" => 1,
    "D=D" => 1,
    "C=N;O=A" => 1,
    "C=M;O=A" => 1,
    "C=S;O=A" => 1,
    "C=D;O=A" => 1,
    "C=N;O=D" => 1,
    "C=M;O=D" => 1,
    "C=S;O=D" => 1,
    "C=D;O=D" => 1);

$safe_string = "\"/[^\p{L}\p{N}
    \p{Arabic}
    \p{Armenian}
    \p{Bengali}
    \p{Bopomofo}
    \p{Braille}
    \p{Buhid}
    \p{Canadian_Aboriginal}
    \p{Cherokee}
    \p{Cyrillic}
    \p{Devanagari}
    \p{Ethiopic}
    \p{Georgian}
    \p{Greek}
    \p{Gujarati}
    \p{Gurmukhi}
    \p{Han}
    \p{Hangul}
    \p{Hanunoo}
    \p{Hebrew}
    \p{Hiragana}
    \p{Inherited}
    \p{Kannada}
    \p{Katakana}
    \p{Khmer}
    \p{Lao}
    \p{Latin}
    \p{Limbu}
    \p{Malayalam}
    \p{Mongolian}
    \p{Myanmar}
    \p{Ogham}
    \p{Oriya}
    \p{Runic}
    \p{Sinhala}
    \p{Syriac}
    \p{Tagalog}
    \p{Tagbanwa}
    \p{Tai_Le}
    \p{Tamil}
    \p{Telugu}
    \p{Thaana}
    \p{Thai}
    \p{Tibetan}
    \p{Yi}
    -.'*\s]/u\"";

/**
 * UTF-8 Case lookup table
 *
 * This lookuptable defines the upper case letters to their correspponding
 * lower case letter in UTF-8
 *
 * @author Andreas Gohr <andi@splitbrain.org>
 */
$utf8_lower_to_upper = array(
    0x0061=>0x0041, 0x03C6=>0x03A6, 0x0163=>0x0162, 0x00E5=>0x00C5, 0x0062=>0x0042,
    0x013A=>0x0139, 0x00E1=>0x00C1, 0x0142=>0x0141, 0x03CD=>0x038E, 0x0101=>0x0100,
    0x0491=>0x0490, 0x03B4=>0x0394, 0x015B=>0x015A, 0x0064=>0x0044, 0x03B3=>0x0393,
    0x00F4=>0x00D4, 0x044A=>0x042A, 0x0439=>0x0419, 0x0113=>0x0112, 0x043C=>0x041C,
    0x015F=>0x015E, 0x0144=>0x0143, 0x00EE=>0x00CE, 0x045E=>0x040E, 0x044F=>0x042F,
    0x03BA=>0x039A, 0x0155=>0x0154, 0x0069=>0x0049, 0x0073=>0x0053, 0x1E1F=>0x1E1E,
    0x0135=>0x0134, 0x0447=>0x0427, 0x03C0=>0x03A0, 0x0438=>0x0418, 0x00F3=>0x00D3,
    0x0440=>0x0420, 0x0454=>0x0404, 0x0435=>0x0415, 0x0449=>0x0429, 0x014B=>0x014A,
    0x0431=>0x0411, 0x0459=>0x0409, 0x1E03=>0x1E02, 0x00F6=>0x00D6, 0x00F9=>0x00D9,
    0x006E=>0x004E, 0x0451=>0x0401, 0x03C4=>0x03A4, 0x0443=>0x0423, 0x015D=>0x015C,
    0x0453=>0x0403, 0x03C8=>0x03A8, 0x0159=>0x0158, 0x0067=>0x0047, 0x00E4=>0x00C4,
    0x03AC=>0x0386, 0x03AE=>0x0389, 0x0167=>0x0166, 0x03BE=>0x039E, 0x0165=>0x0164,
    0x0117=>0x0116, 0x0109=>0x0108, 0x0076=>0x0056, 0x00FE=>0x00DE, 0x0157=>0x0156,
    0x00FA=>0x00DA, 0x1E61=>0x1E60, 0x1E83=>0x1E82, 0x00E2=>0x00C2, 0x0119=>0x0118,
    0x0146=>0x0145, 0x0070=>0x0050, 0x0151=>0x0150, 0x044E=>0x042E, 0x0129=>0x0128,
    0x03C7=>0x03A7, 0x013E=>0x013D, 0x0442=>0x0422, 0x007A=>0x005A, 0x0448=>0x0428,
    0x03C1=>0x03A1, 0x1E81=>0x1E80, 0x016D=>0x016C, 0x00F5=>0x00D5, 0x0075=>0x0055,
    0x0177=>0x0176, 0x00FC=>0x00DC, 0x1E57=>0x1E56, 0x03C3=>0x03A3, 0x043A=>0x041A,
    0x006D=>0x004D, 0x016B=>0x016A, 0x0171=>0x0170, 0x0444=>0x0424, 0x00EC=>0x00CC,
    0x0169=>0x0168, 0x03BF=>0x039F, 0x006B=>0x004B, 0x00F2=>0x00D2, 0x00E0=>0x00C0,
    0x0434=>0x0414, 0x03C9=>0x03A9, 0x1E6B=>0x1E6A, 0x00E3=>0x00C3, 0x044D=>0x042D,
    0x0436=>0x0416, 0x01A1=>0x01A0, 0x010D=>0x010C, 0x011D=>0x011C, 0x00F0=>0x00D0,
    0x013C=>0x013B, 0x045F=>0x040F, 0x045A=>0x040A, 0x00E8=>0x00C8, 0x03C5=>0x03A5,
    0x0066=>0x0046, 0x00FD=>0x00DD, 0x0063=>0x0043, 0x021B=>0x021A, 0x00EA=>0x00CA,
    0x03B9=>0x0399, 0x017A=>0x0179, 0x00EF=>0x00CF, 0x01B0=>0x01AF, 0x0065=>0x0045,
    0x03BB=>0x039B, 0x03B8=>0x0398, 0x03BC=>0x039C, 0x045C=>0x040C, 0x043F=>0x041F,
    0x044C=>0x042C, 0x00FE=>0x00DE, 0x00F0=>0x00D0, 0x1EF3=>0x1EF2, 0x0068=>0x0048,
    0x00EB=>0x00CB, 0x0111=>0x0110, 0x0433=>0x0413, 0x012F=>0x012E, 0x00E6=>0x00C6,
    0x0078=>0x0058, 0x0161=>0x0160, 0x016F=>0x016E, 0x03B1=>0x0391, 0x0457=>0x0407,
    0x0173=>0x0172, 0x00FF=>0x0178, 0x006F=>0x004F, 0x043B=>0x041B, 0x03B5=>0x0395,
    0x0445=>0x0425, 0x0121=>0x0120, 0x017E=>0x017D, 0x017C=>0x017B, 0x03B6=>0x0396,
    0x03B2=>0x0392, 0x03AD=>0x0388, 0x1E85=>0x1E84, 0x0175=>0x0174, 0x0071=>0x0051,
    0x0437=>0x0417, 0x1E0B=>0x1E0A, 0x0148=>0x0147, 0x0105=>0x0104, 0x0458=>0x0408,
    0x014D=>0x014C, 0x00ED=>0x00CD, 0x0079=>0x0059, 0x010B=>0x010A, 0x03CE=>0x038F,
    0x0072=>0x0052, 0x0430=>0x0410, 0x0455=>0x0405, 0x0452=>0x0402, 0x0127=>0x0126,
    0x0137=>0x0136, 0x012B=>0x012A, 0x03AF=>0x038A, 0x044B=>0x042B, 0x006C=>0x004C,
    0x03B7=>0x0397, 0x0125=>0x0124, 0x0219=>0x0218, 0x00FB=>0x00DB, 0x011F=>0x011E,
    0x043E=>0x041E, 0x1E41=>0x1E40, 0x03BD=>0x039D, 0x0107=>0x0106, 0x03CB=>0x03AB,
    0x0446=>0x0426, 0x00FE=>0x00DE, 0x00E7=>0x00C7, 0x03CA=>0x03AA, 0x0441=>0x0421,
    0x0432=>0x0412, 0x010F=>0x010E, 0x00F8=>0x00D8, 0x0077=>0x0057, 0x011B=>0x011A,
    0x0074=>0x0054, 0x006A=>0x004A, 0x045B=>0x040B, 0x0456=>0x0406, 0x0103=>0x0102,
    0x03BB=>0x039B, 0x00F1=>0x00D1, 0x043D=>0x041D, 0x03CC=>0x038C, 0x00E9=>0x00C9,
    0x00F0=>0x00D0, 0x0457=>0x0407, 0x0123=>0x0122
);

// Common text
$com_txt_langs = array(
    array ("en","English"),
    array ("sq","Albanian"),
    array ("am","Amharic"),
    array ("ar","Arabic"),
    array ("bn","Bengali"),
    array ("bg","Bulgarian"),
    array ("ca","Catalan"),
    array ("zh-cn","Chinese-Simplified"),
    array ("zh-tw","Chinese-Traditional"),
    array ("hr","Croatian"),
    array ("cs","Czech"),
    array ("da","Danish"),
    array ("nl","Dutch"),
    array ("et","Estonian"),
    array ("fa","Farsi"),
    array ("fi","Finnish"),
    array ("fr","French"),
    array ("de","German"),
    array ("el","Greek"),
    array ("hi","Hindi"),
    array ("hu","Hungarian"),
    array ("it","Italian"),
    array ("ja","Japanese"),
    array ("lv","Latvian"),
    array ("no","Norwegian"),
    array ("pl","Polish"),
    array ("pt","Portuguese"),
    array ("ro","Romanian"),
    array ("ru","Russian"),
    array ("sr","Serbian"),
    array ("sk","Slovak"),
    array ("sl","Slovenian"),
    array ("es","Spanish"),
    array ("sw","Swahili"),
    array ("sv","Swedish"),
    array ("tr","Turkish")
);

$common = array();

foreach ($com_txt_langs as $id => $code) {
    $lines = @file($include_dir.'/common/'.$code[0].'_common.txt');
    if (is_array($lines)) {
        foreach ($lines as $id => $word) {
            $common[$code[0]][trim($word)] = 1;
        }
    }
}

// Rxtensions
$ext = array();

$lines = @file('ext.txt');

if (is_array($lines)) {
    foreach ($lines as $id => $word) {
        $ext[] = trim($word);
    }
}


/**
 * Returns a new class instance for stemming
 *
 * @param string $language Language to stem_class
 *
 * @return class instance
 */
function stemInstance($language)
{
    switch ($language) {
    case "Danish":
        $stem = new Danish();
        break;
    case "Dutch":
        $stem = new Dutch();
        break;
    case "French":
        $stem = new French();
        break;
    case "German":
        $stem = new German();
        break;
    case "Italian":
        $stem = new Italian();
        break;
    case "Norwegian":
        $stem = new Norwegian();
        break;
    case "Portuguese":
        $stem = new Portuguese();
        break;
    case "Romanian":
        $stem = new Romanian();
        break;
    case "Russian":
        $stem = new Russian();
        break;
    case "Spanish":
        $stem = new Spanish();
        break;
    case "Swedish":
        $stem = new Swedish();
        break;
    default:
        $stem = new English();
    }

    return $stem;
}

/**
 * Returns the result of a query as an array
 *
 * @param string $query SQL query string
 *
 * @return array | null array
 */
function sqlFetchAll($query)
{
    global $db;

    $stmt = $db->prepare($query);
    if ($stmt) {
        $stmt->execute() or die("Execution failed: ".$stmt->error);
        $result = $stmt->get_result();
        while ($row = $result->fetch_array(MYSQLI_ASSOC)) {
            $data[] = $row;
        }
        $stmt->close();
    } else {
        trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
    }

    return $data;
}


/**
 * Removes duplicate elements from an array
 *
 * @param array $arr Array to process
 *
 * @return array $newarry Array with duplicates removed
 */
function distinctArray($arr)
{
    rsort($arr);
    reset($arr);
    $newarr = array();
    $i = 0;
    $element = current($arr);

    for ($n = 0; $n < sizeof($arr); $n++) {
        if (next($arr) != $element) {
            $newarr[$i] = $element;
            $element = current($arr);
            $i++;
        }
    }

    return $newarr;
}


/**
 * Get categories
 *
 * @param int $parent Id of parent category
 *
 * @return array $arr Array of categories with designated parent
 */
function getCats($parent)
{
    global $mysql_table_prefix, $db;

    if (!is_numeric($parent)) {
        $parent = 0;
    }
    $query = "SELECT * FROM ".$mysql_table_prefix
      ."categories WHERE parent_num=?";
    $stmt = $db->prepare($query);
    if ($stmt) {
        $stmt->bind_param("i", $parent);
        $stmt->execute() or die("Execution failed: ".$stmt->error);
        $result = $stmt->get_result();
        $arr[] = $parent;
        while ($row = $result->fetch_array(MYSQLI_ASSOC)) {
            $id = $row['category_id'];
            $arr = addArrays($arr, getCats($id));
        }
        $stmt->close();
    } else {
        trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
    }

    return $arr;
}


/**
 * Combine arrays
 *
 * @param array $arr1 First array
 * @param array $arr2 Second array
 *
 * @return array $arr1 Combined array
 */
function addArrays($arr1, $arr2)
{
    foreach ($arr2 as $elem) {
        $arr1[] = $elem;
    }
    return $arr1;
}

/**
 * Get HTTP variables
 *
 * @return array $httpvars Array of http variables
 */
function getHttpVars()
{
    $superglobs = array(
      '_POST',
      '_GET',
      'HTTP_POST_VARS',
      'HTTP_GET_VARS');

    $httpvars = array();

    // extract the right array
    foreach ($superglobs as $glob) {
        global $$glob;
        if (isset($$glob) && is_array($$glob)) {
            $httpvars = $$glob;
        }
        if (count($httpvars) > 0) {
            break; 
        }
    }
    return $httpvars;

}


/**
 * Get the count of a substring within a string
 *
 * @param string $haystack Full string
 * @param string $needle   The substring
 *
 * @return int $count The numeric result
 */
function countSubStrs($haystack, $needle)
{
    $count = 0;
    while (mb_strpos($haystack, $needle, 0, 'utf-8') !== false) {
        $haystack = mb_substr(
            $haystack ?? '', (
            mb_strpos($haystack, $needle, 0, 'utf-8') + 1
            ), null, 'utf-8'
        );
        $count++;
    }
        return $count;
}


/**
 * Replace single and double quotes
 *
 * @param string $str The string to be processed
 *
 * @return string $str The modified string
 */
function quoteReplace($str)
{
        $str = str_replace("\"", "&quot;", $str);
        return str_replace("'", "&apos;", $str);
}


/**
 * Get directory contents
 *
 * @param string $dir The directory
 *
 * @return array $contents Array of contents
 */
function getDirContents($dir)
{
    if (is_null($dir)) {
        $dir = "";
    }
    $contents = Array();
    if ($handle = opendir($dir)) {
        while (false !== ($file = readdir($handle))) {
            if ($file != "." && $file != "..") {
                  $contents[] = $file;
            }
        }
        closedir($handle);
    }
        sort($contents);
        return $contents;
}


/**
 * Replace '&' in a string
 *
 * @param string $str The string to be modified
 *
 * @return string $str The modified string
 */
function replaceAmpersand($str)
{
        return str_replace("&", "%26", $str);
}


/**
 * Check if the url gets redirected
 *
 * @param string $url The url being checked
 *
 * @return string $url The url checked and possibly updated
 */
function checkRedirect($url) 
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_HEADER, true);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_URL, $url);
    $out = curl_exec($ch);

    // line endings is the wonkiest piece of this whole thing
    $out = str_replace("\r", "", $out);

    // only look at the headers
    $headers_end = mb_strpos($out, "\n\n", 0, 'utf-8');
    if ($headers_end !== false ) {
        $out = mb_substr($out ?? '', 0, $headers_end, 'utf-8');
    }

    $headers = explode("\n", $out);
    foreach ($headers as $header) {
        if (mb_substr($header ?? '', 0, 10, 'utf-8') == "Location: " ) {
            $target = mb_substr($header ?? '', 10, null, 'utf-8');
            $url = $target;
        }
    }

    curl_close($ch);

    return $url;
}

/**
 * This function returns any UTF-8 encoded text as a list of Unicode values.
 * Based upon an earlier work by: Scott Michael Reynen <scott@randomchaos.com>
 *
 * @param string $str Pointer to UTF-8 string
 *
 * @return string Unicode version of the string
 */
function utf8ToUnicode(&$str)
{
    $unicode = array();
    $values = array();
    $looking_for = 1;
    for ($i = 0; $i < strlen($str); $i++) {
        $this_value = ord($str[$i]);
        if ($this_value < 128) {
            $unicode[] = $this_value;
        } else {
            if (count($values) == 0) {
                $looking_for = ($this_value < 224)?2:3;
            }
            $values[] = $this_value;
            if (count($values) == $looking_for) {
                $number = ($looking_for == 3) ?
                (($values[0] % 16) * 4096) + (($values[1] % 64) * 64)
                 + ($values[2] % 64):
                (($values[0] % 32) * 64) + ($values[1] % 64 );
                $unicode[] = $number;
                $values = array();
                $looking_for = 1;
            }
        }
    }
    return $unicode;
}

/**
 * This function converts a Unicode array back to its UTF-8 representation
 * Based upon an earlier work by: Scott Michael Reynen <scott@randomchaos.com>
 *
 * @param string $str Pointer to Unicode array
 *
 * @return string UTF-8 string
 */
function unicodeToUtf8(&$str)
{
    if (!is_array($str)) {
        return '';
    }
    $utf8 = '';
    foreach ($str as $unicode) {
        if ($unicode < 128 ) {
                $utf8 .= chr($unicode);
        } elseif ($unicode < 2048) {
            $utf8 .= chr(192 + (($unicode - ($unicode % 64)) / 64));
            $utf8 .= chr(128 + ($unicode % 64));
        } else {
            $utf8 .= chr(224 + (($unicode - ($unicode % 4096)) / 4096));
            $utf8 .= chr(128 + ((($unicode % 4096) - ($unicode % 64)) / 64));
            $utf8 .= chr(128 + ($unicode % 64));
        }
    }
    return $utf8;
}

/**
 * Replacement for deprecated rand() and mt_rand()
 * 
 * @param int $min Minimum seed integer
 * @param int $max Maximum seed integer
 * 
 * @return int Random integer
 */
function My_Mt_rand($min = null, $max = null)
{
    if (floatval(mb_substr(phpversion(), 0, 3)) < 8.2) {
        return rand($min, $max);
    } else {
        if (!isset($GLOBALS['my_mt_rand'])) {
            $GLOBALS['my_mt_rand']
                = new \Random\Randomizer(new \Random\Engine\Mt19937());
        }
 
        if ($min === null && $max === null) {
            return $GLOBALS['my_mt_rand']->nextInt();
        }
 
        return $GLOBALS['my_mt_rand']->getInt($min, $max);
    }
}

/**
 * Selector common to editSiteScreen and indexScreen
 * 
 * @return null
 */
function commonTextLang()
{
    global $row, $ctlang;

    echo "<SELECT name='ctlang'>
    <option value = 'en' ";
    if ($row['ct_lang'] == 'en') {
        echo "selected";
    }
    echo ">English</option>
    <option value = 'sq' ";
    if ($row['ct_lang'] == 'sq') {
        echo "selected";
    }
    echo ">Albanian</option>
    <option value = 'am' ";
    if ($row['ct_lang'] == 'am') {
        echo "selected";
    }
    echo ">Amharic</option>
    <option value = 'ar' ";
    if ($row['ct_lang'] == 'ar') {
        echo "selected";
    }
    echo ">Arabic</option>
    <option value = 'bn' ";
    if ($row['ct_lang'] == 'bn') {
        echo "selected";
    }
    echo ">Bengali</option>
    <option value = 'bg' ";
    if ($row['ct_lang'] == 'bg') {
        echo "selected";
    }
    echo ">Bulgarian</option>
    <option value = 'ca' ";
    if ($row['ct_lang'] == 'ca') {
        echo "selected";
    }
    echo ">Catalan</option>
    <option value = 'zh-cn' ";
    if ($row['ct_lang'] == 'zh-cn') {
        echo "selected";
    }
    echo ">Chinese-Simplified</option>
    <option value = 'zh-tw' ";
    if ($row['ct_lang'] == 'zh-tw') {
        echo "selected";
    }
    echo ">Chinese-Traditional</option>
    <option value = 'hr' ";
    if ($row['ct_lang'] == 'hr') {
        echo "selected";
    }
    echo ">Croatian</option>
    <option value = 'cs' ";
    if ($row['ct_lang'] == 'cs') {
        echo "selected";
    }
    echo ">Czech</option>
    <option value = 'da' ";
    if ($row['ct_lang'] == 'da') {
        echo "selected";
    }
    echo ">Danish</option>
    <option value = 'nl' ";
    if ($row['ct_lang'] == 'nl') {
        echo "selected";
    }
    echo ">Dutch</option>
    <option value = 'et' ";
    if ($row['ct_lang'] == 'et') {
        echo "selected";
    }
    echo ">Estonian</option>
    <option value = 'fa' ";
    if ($row['ct_lang'] == 'fa') {
        echo "selected";
    }
    echo ">Farsi</option>
    <option value = 'fi' ";
    if ($row['ct_lang'] == 'fi') {
        echo "selected";
    }
    echo ">Finnish</option>
    <option value = 'fr' ";
    if ($row['ct_lang'] == 'fr') {
        echo "selected";
    }
    echo ">French</option>
    <option value = 'de' ";
    if ($row['ct_lang'] == 'de') {
        echo "selected";
    }
    echo ">German</option>
    <option value = 'el' ";
    if ($row['ct_lang'] == 'el') {
        echo "selected";
    }
    echo ">Greek</option>
    <option value = 'hi' ";
    if ($row['ct_lang'] == 'hi') {
        echo "selected";
    }
    echo ">Hindi</option>
    <option value = 'hu' ";
    if ($row['ct_lang'] == 'hu') {
        echo "selected";
    }
    echo ">Hungarian</option>
    <option value = 'it' ";
    if ($row['ct_lang'] == 'it') {
        echo "selected";
    }
    echo ">Italian</option>
    <option value = 'ja' ";
    if ($row['ct_lang'] == 'ja') {
        echo "selected";
    }
    echo ">Japanese</option>
    <option value = 'lv' ";
    if ($row['ct_lang'] == 'lv') {
        echo "selected";
    }
    echo ">Latvian</option>
    <option value = 'no' ";
    if ($row['ct_lang'] == 'no') {
        echo "selected";
    }
    echo ">Norwegian</option>
    <option value = 'pl' ";
    if ($row['ct_lang'] == 'pl') {
        echo "selected";
    }
    echo ">Polish</option>
    <option value = 'pt' ";
    if ($row['ct_lang'] == 'pt') {
        echo "selected";
    }
    echo ">Portuguese</option>
    <option value = 'ro' ";
    if ($row['ct_lang'] == 'ro') {
        echo "selected";
    }
    echo ">Romanian</option>
    <option value = 'ru' ";
    if ($row['ct_lang'] == 'ru') {
        echo "selected";
    }
    echo ">Russian</option>
    <option value = 'sr' ";
    if ($row['ct_lang'] == 'sr') {
        echo "selected";
    }
    echo ">Serbian</option>
    <option value = 'sk' ";
    if ($row['ct_lang'] == 'sk') {
        echo "selected";
    }
    echo ">Slovak</option>
    <option value = 'sl' ";
    if ($row['ct_lang'] == 'sl') {
        echo "selected";
    }
    echo ">Slovenian</option>
    <option value = 'es' ";
    if ($row['ct_lang'] == 'es') {
        echo "selected";
    }
    echo ">Spanish</option>
    <option value = 'sw' ";
    if ($row['ct_lang'] == 'sw') {
        echo "selected";
    }
    echo ">Swahili</option>
    <option value = 'sv' ";
    if ($row['ct_lang'] == 'sv') {
        echo "selected";
    }
    echo ">Swedish</option>
    <option value = 'tr' ";
    if ($row['ct_lang'] == 'tr') {
        echo "selected";
    }
    echo ">Turkish</option>
    </SELECT> Common text language<br>";
}
