<?php
/**
 * SphiderLite Version 2.6.*
 * Sphider is licensed under the GNU GPL.
 *
 * PHP version 5.4
 *
 * @category Sphider_Search
 * @package  Search
 * @author   R. Prew  <CaptQ@blog.worldspaceflight.com>
 * @license  GNU GPL
 * @link     http://www.sphidersearch.com
 *
 * Original base code by:
 *   Ando Saabas  ando(a t)set.ee
 */
//error_reporting(E_ERROR | E_DEPRECATED); //Development only
error_reporting(0);

/**
 * Swap Max in array
 *
 * @param array  $arr    The array to affect
 * @param int    $start  Starting position
 * @param string $domain Domain name
 *
 * @return void
 */
function swapMax(&$arr, $start, $domain)
{
    $pos  = $start;
    $maxweight = $arr[$pos]['weight'];
    for ($i = $start; $i< count($arr); $i++) {
        if ($arr[$i]['domain'] == $domain) {
            $pos = $i;
            $maxweight = $arr[$i]['weight'];
            break;
        }
        if ($arr[$i]['weight'] > $maxweight) {
            $pos = $i;
            $maxweight = $arr[$i]['weight'];
        }
    }
    $temp = $arr[$start];
    $arr[$start] = $arr[$pos];
    $arr[$pos] = $temp;
}


/**
 * Sort with domains
 *
 * @param array $arr The array to sort
 *
 * @return void
 */
function sortWithDomains(&$arr)
{
    $domain = -1;
    for ($i = 0; $i< count($arr)-1; $i++) {
        swapMax($arr, $i, $domain);
        $domain = $arr[$i]['domain'];
    }
}


/**
 * Compare arrays
 *
 * @param array $a First array
 * @param array $b Second array
 *
 * @return int Numeric indication of comparison results
 */
function cmp($a, $b)
{
    if ($a['weight'] == $b['weight']) {
        return 0; 
    }

    return ($a['weight'] > $b['weight']) ? -1 : 1;
}


/**
 * Make a boolean list
 *
 * @param string $a String to parse
 *
 * @return array $returnWords Array of words
 */
function makeBoolList($a)
{
    global $entities, $stem_words, $stem;
    global $special_characters_array;

    foreach ($entities as $key => $char) {
        $a = preg_replace("/".$key."/i", $char, $a ?? '');
    }
    $a = trim($a);
    $a = preg_replace("/&quot;/i", "\"", $a ?? '');
    $returnWords = array();
    //get all phrases
    $regs = Array();
    while (preg_match("/([-]?)\"([^\"]+)\"/", $a ?? '', $regs)) {
        if ($regs[1] == '') {
            $returnWords['+s'][] = $regs[2];
            $returnWords['hilight'][] = $regs[2];
        } else {
            $returnWords['-s'][] = $regs[2];
        }
        $a = str_replace($regs[0], "", $a);
    }
    $a = mb_strtolower(preg_replace("/[ ]+/", " ", $a ?? ''));
    $a = trim($a);
    $words = explode(' ', $a);
    if ($a=="") {
        $limit = 0;
    } else {
        $limit = count($words);
    }

    $k = 0;
    //get all words (both include and exlude)
    $includeWords = array();
    while ($k < $limit) {
        if (mb_substr($words[$k] ?? '', 0, 1, 'utf-8') == '+') {
            $includeWords[] = mb_substr($words[$k] ?? '', 1, null, 'utf-8');
            if (!ignoreWord(mb_substr($words[$k] ?? '', 1, null, 'utf-8'))) {
                 $returnWords['hilight'][]
                     = mb_substr($words[$k] ?? '', 1, null, 'utf-8');
                if ($stem_words == 1) {
                    $returnWords['hilight'][]
                        = $stem->stem(mb_substr($words[$k] ?? '', 1, null, 'utf-8'));
                }
            }
        } else if (mb_substr($words[$k] ?? '', 0, 1, 'utf-8') == '-') {
            $returnWords['-'][] = mb_substr($words[$k] ?? '', 1, null, 'utf-8');
        } else {
            $includeWords[] = $words[$k];
            if (!ignoreWord($words[$k])) {
                $returnWords['hilight'][] = $words[$k];
                if ($stem_words == 1) {
                     $returnWords['hilight'][] = $stem->stem($words[$k]);
                }
            }
        }
        $k++;
    }
    //add words from phrases to includes
    if (isset($returnWords['+s'])) {
        foreach ($returnWords['+s'] as $phrase) {
            $phrase = mb_strtolower(
                preg_replace("/[ ]+/", " ", $phrase ?? '') ?? '', 'utf-8'
            );
            $phrase = trim($phrase);
            $temparr = explode(' ', $phrase);
            foreach ($temparr as $w) {
                $includeWords[] = $w; 
            }
        }
    }

    foreach ($includeWords as $word) {
        $word = str_replace($special_characters_array, '', $word);
        if (!($word =='')) {
            if (ignoreWord($word)) {
                $returnWords['ignore'][] = $word;
            } else {
                $returnWords['+'][] = $word;
            }
        }
    }

    return $returnWords;

}


/**
 * Determine if a word should be ignored
 *
 * @param string $word The word in question
 *
 * @return bool Returns 0 or 1
 */
function ignoreWord($word) 
{
    global $common, $min_word_length, $index_numbers;

    if ($index_numbers == 1) {
        if ($index_decimals == 0) {
            $pattern = "/[\pL0-9]+/i";
        } else {
            $pattern = "/[\pL\d]+/i";
        }
    } else {
        $pattern = "/[\pL]+/i";
    }
    if (mb_strlen($word, 'utf-8') < $min_word_length
        || (!preg_match($pattern, $word ?? ''))
        || ($common[$word] == 1)
    ) {
        return 1;
    } else {
        return 0;
    }
}


/**
 * Search function
 *
 * @param string $searchstr The search string
 * @param int    $category  Is this a category search
 * @param int    $start     Starting place for search results set
 * @param int    $per_page  Maximum number of results in this set
 * @param string $type      Type of search
 * @param int    $domain    Domain being searched
 *
 * @return array $res Results array
 */
function search($searchstr, $category, $start, $per_page, $type, $domain)
{
    global $length_of_link_desc,$mysql_table_prefix, $show_meta_description;
    global $merge_site_results, $stem_words, $did_you_mean_enabled, $db;
    global $bound_search_result, $stem, $qfloor;

    $possible_to_find = 1;
    $p1 = $db->real_escape_string($domain ?? '');
    $query = "SELECT domain_id FROM ".$mysql_table_prefix
      ."domains WHERE domain = ? ";
    $stmt = $db->prepare($query);
    if ($stmt) {
        $stmt->bind_param("s", $p1);
        $stmt->execute() or die("Execution failed: ".$stmt->error);
        $stmt->bind_result($domain_id);
        $stmt->store_result();
        $num_rows = $stmt->num_rows;
        if ($num_rows > 0) {
            $stmt->fetch();
            $domain_qry = "AND domain = '".$domain_id."'";
        } else {
            $domain_qry = "";
        }
        $stmt->free_result();
        $stmt->close();
    } else {
        trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
    }

    //find all sites that should not be included in the result
    // PHP7.2+ distinguishes between null and empty
    if (is_null($searchstr['+']) || count($searchstr['+']) == 0) {
        return null;
    }
    $wordarray = $searchstr['-'];
    // PHP7.2+ distinguishes between null and empty
    if (is_null($wordarray)) {
        $wordarray = [];
    }
    $notlist = array();
    $not_words = 0;
    while ($not_words < count($wordarray)) {
        if ($stem_words == 1) {
            $searchword = addslashes($stem->stem($wordarray[$not_words]));
        } else {
            $searchword = addslashes($wordarray[$not_words]);
        }
        $wordmd5 = mb_substr(md5($searchword) ?? '', 0, 1, 'utf-8');

        $query1 = "SELECT link_id FROM ".$mysql_table_prefix."link_keyword"
          .$wordmd5.", ".$mysql_table_prefix."keywords WHERE "
          .$mysql_table_prefix."link_keyword".$wordmd5.".keyword_id = "
          .$mysql_table_prefix."keywords.keyword_id AND keyword = ?";
        $stmt1 = $db->prepare($query1);
        if ($stmt1) {
            $stmt1->bind_param("s", $searchword);
            $stmt1->execute() or die("Execution failed: ".$stmt1->error);
            $result = $stmt1->get_result();
            $stmt1->close();
        } else {
            trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
        }
        while ($row = $result->fetch_array(MYSQLI_NUM)) {
            $notlist[$not_words]['id'][$row[0]] = 1;
        }
        $not_words++;
    }

    //find all sites containing the search phrase
    $wordarray = $searchstr['+s'];
    // PHP7.2+ distinguishes between null and empty
    if (is_null($wordarray)) {  
        $wordarray = [];
    }
    $phrase_words = 0;
    while ($phrase_words < count($wordarray)) {
        $searchword = addslashes($wordarray[$phrase_words]);
        $searchwordlike = "%".$searchword."%";
        $query1 = "SELECT link_id FROM ".$mysql_table_prefix
          ."links WHERE FULLTXT LIKE ?";
        $stmt1 = $db->prepare($query1);
        if ($stmt1) {
            $stmt1->bind_param("s", $searchwordlike);
            $stmt1->execute() or die("Execution failed: ".$stmt1->error);
            $stmt1->bind_result($link_id);
            $stmt1->store_result();
            $num_rows = $stmt1->num_rows;
            if ($num_rows == 0) {
                $possible_to_find = 0;
                break;
            }
            while ($stmt1->fetch()) {
                $phraselist[$phrase_words]['id'][$link_id] = 1;
            }
            $stmt1->free_result();
            $stmt1->close();
        } else {
            trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
        }
        $phrase_words++;
    }
    if (($category> 0) && $possible_to_find==1) {
        $allcats = getCats($category);
        $catlist = implode(",", $allcats);
        $query1 = "SELECT link_id FROM ".$mysql_table_prefix."links, "
          .$mysql_table_prefix."sites, ".$mysql_table_prefix."categories, "
          .$mysql_table_prefix."site_category WHERE ".$mysql_table_prefix
          ."links.site_id = ".$mysql_table_prefix."sites.site_id AND "
          .$mysql_table_prefix."sites.site_id = ".$mysql_table_prefix
          ."site_category.site_id AND ".$mysql_table_prefix
          ."site_category.category_id IN (".$db->real_escape_string($catlist).")";
        $stmt1 = $db->prepare($query1);
        if ($stmt1) {
            $stmt1->execute() or die("Execution failed: ".$stmt1->error);
            $stmt1->bind_result($link_id);
            $stmt1->store_result();
            $num_rows = $stmt1->num_rows;
            if ($num_rows == 0) {
                $possible_to_find = 0;
            }
            while ($stmt1->fetch()) {
                $category_list[$link_id] = 1;
            }
            $stmt1->free_result();
            $stmt1->close();
        } else {
            trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
        }
    }

    // Core wildcard (*) code originally conceived by Tec in 2007,
    // adapted for MySQLi extension
    // Find all sites that include the search word
    $wordarray = $searchstr['+'];
    // PHP7.2+ distinguishes between null and empty
    if (is_null($wordarray)) {  
        $wordarray = [];
    }
    $words = 0;
    $starttime = getMicroTime();

    //  If * wildcard, search for part of a word
    $wild_correct = 0;
    // get only the first word of the search query
    $searchword = addslashes($wordarray[$words]);
    $wildcount = mb_substr_count($searchword, '*', 'utf-8');

    // if * wildcard, enter here
    if ($wildcount) {
        $searchword = str_replace('*', '%', $searchword);
        $query = "SELECT keyword_id, keyword FROM ".$mysql_table_prefix
          ."keywords WHERE keyword LIKE ?";
        $stmt = $db->prepare($query);
        if ($stmt) {
            $stmt->bind_param("s", $searchword);
            $stmt->execute() or die("Execution failed: ".$stmt->error);
            $stmt->bind_result($keyword_id, $keyword);
            $stmt->store_result();
            $num_rows = $stmt->num_rows;
            if ($num_rows == 0) {
                // if there was no searchword in table keywords
                $possible_to_find = 0;
                $break = 1;
            }
            if ($num_rows != 0) {
                global $all_wild;
                $all_wild = '';
                for ($i=0; $i<$num_rows;$i++) {
                    // get all searchwords as keywords from table keywords
                    $stmt->data_seek($i);
                    $stmt->fetch();
                    $all_wild = ("$all_wild $keyword");
                    // calculate attribute for link_keyword table
                    $wordmd5 = mb_substr(md5($keyword) ?? '', 0, 1, 'utf-8');
                    $query1 = "SELECT link_id, weight, domain FROM "
                      .$mysql_table_prefix."link_keyword".$wordmd5
                      ." WHERE keyword_id = ? ".$domain_qry
                      ." ORDER BY weight DESC";
                     $stmt1 = $db->prepare($query1);
                    if ($stmt1) {
                        $stmt1->bind_param("i", $keyword_id);
                        $stmt1->execute()
                          or die("Execution failed: ".$stmt1->error);
                        $stmt1->bind_result($link_id, $weight, $domain);
                        $stmt1->store_result();
                        $lines = $stmt1->num_rows;
                        if ($lines != 0) {
                             $indx = $words;
                        }
                        while ($stmt1->fetch()) {
                            $linklist[$indx]['id'][] = $link_id;
                            $domains[$link_id] = $domain;
                            $linklist[$indx]['weight'][$link_id] = $weight;
                        }
                        $stmt1->free_result();
                        $stmt1->close();
                    } else {
                        trigger_error(
                            'Statement failed : '
                            .$db->error, E_USER_ERROR
                        );
                    }
                }
            }
            $stmt->free_result();
            $stmt->close();
        } else {
             trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
        }
        // If no wildcard, enter here
    } else {
        while (($words < count($wordarray)) && $possible_to_find == 1) {
            if ($stem_words == 1) {
                $searchword = $stem->stem($wordarray[$words]);
            } else {
                $searchword = $wordarray[$words];
            }
            $wordmd5 = mb_substr(md5($searchword) ?? '', 0, 1, 'utf-8');
            $query1 = "SELECT distinct link_id, weight, domain FROM "
              .$mysql_table_prefix."link_keyword".$wordmd5.", "
              .$mysql_table_prefix."keywords WHERE ".$mysql_table_prefix
              ."link_keyword".$wordmd5.".keyword_id = ".$mysql_table_prefix
              ."keywords.keyword_id AND keyword = ? ".$domain_qry
              ." ORDER BY weight DESC";
            $stmt1 = $db->prepare($query1);
            if ($stmt1) {
                $stmt1->bind_param("s", $searchword);
                $stmt1->execute() or die("Execution failed: ".$stmt1->error);
                $stmt1->bind_result($link_id, $weight, $domain);
                $stmt1->store_result();
                $num_rows = $stmt1->num_rows;
                if ($num_rows == 0) {
                    if ($type != "or") {
                        $possible_to_find = 0;
                        break;
                    }
                }
                if ($type == "or") {
                    $indx = 0;
                } else {
                    $indx = $words;
                }

                while ($stmt1->fetch()) {
                    $linklist[$indx]['id'][] = $link_id;
                    $domains[$link_id] = $domain;
                    $linklist[$indx]['weight'][$link_id] = $weight;
                }
                $stmt1->free_result();
                $stmt1->close();
            } else {
                trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
            }
            $words++;
        }
    }
    // End of wild card code

    if ($type == "or") {
        $words = 1;
    }
    $result_array_full = Array();

    if ($possible_to_find !=0) {
        if ($words == 1 && $not_words == 0 && $category < 1) {
            //if there is only one search word, we already have the result
            $result_array_full = $linklist[0]['weight'];
        } else { //otherwise build an intersection of all the results
            $j= 1;
            $min = 0;
            while ($j < $words) {
                if (count($linklist[$min]['id']) > count($linklist[$j]['id'])) {
                    $min = $j;
                }
                $j++;
            }

            $j = 0;
            $temp_array = $linklist[$min]['id'];
            $count = 0;
            while ($j < count($temp_array)) {
                $k = 0; //and word counter
                $n = 0; //not word counter
                $o = 0; //phrase word counter
                $weight = 1;
                $break = 0;
                while ($k < $words && $break== 0) {
                    if ($linklist[$k]['weight'][$temp_array[$j]] > 0) {
                        $weight = $weight + $linklist[$k]['weight'][$temp_array[$j]];
                    } else {
                           $break = 1;
                    }
                    $k++;
                }
                while ($n < $not_words && $break== 0) {
                    if ($notlist[$n]['id'][$temp_array[$j]] > 0) {
                        $break = 1;
                    }
                    $n++;
                }

                while ($o < $phrase_words && $break== 0) {
                    if ($phraselist[$n]['id'][$temp_array[$j]] != 1) {
                        $break = 1;
                    }
                    $o++;
                }
                if ($break== 0 && $category > 0
                    && $category_list[$temp_array[$j]] != 1
                ) {
                    $break = 1;
                }

                if ($break == 0) {
                    $result_array_full[$temp_array[$j]] = $weight;
                    $count ++;
                }
                $j++;
            }
        }
    }
    $end = getMicroTime()- $starttime;

    if ((count($result_array_full) == 0 || $possible_to_find == 0)
        && $did_you_mean_enabled == 1
    ) {
        reset($searchstr['+']);
        foreach ($searchstr['+'] as $word) {
            $word = addslashes($word);
            $max_distance = 100;
            $near_word ="";
            $query = "SELECT keyword FROM ".$mysql_table_prefix
              ."keywords WHERE SOUNDEX(keyword) = SOUNDEX( ? )";
            $stmt = $db->prepare($query);
            if ($stmt) {
                $stmt->bind_param("s", $word);
                $stmt->execute() or die("Execution failed: ".$stmt->error);
                $stmt->bind_result($keyword);
                while ($stmt->fetch()) {
                    $distance = levenshtein($keyword, $word);
                    if ($distance < $max_distance && $distance <4) {
                        $max_distance = $distance;
                        $near_word = $keyword;
                    }
                }
                $stmt->close();
            } else {
                trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
            }
            if ($near_word != "" && $word != $near_word) {
                $near_words[$word] = $near_word;
            }
        }
        $res['did_you_mean'] = $near_words;
        return $res;
    }
    if (count($result_array_full) == 0) {
        return null;
    }
    if ($qfloor > 0) {
        $maxval = max($result_array_full);
        foreach ($result_array_full as $key => $value) {
            if (number_format($value/$maxval*100, 2) < $qfloor) {
                unset($result_array_full[$key]);
            }
        }
    }
    arsort($result_array_full);

    if ($merge_site_results == 1 && $domain_qry == "") {
        foreach ($result_array_full as $key => $value) {
            if (!isset($domains_to_show[$domains[$key]])) {
                $result_array_temp[$key] = $value;
                $domains_to_show[$domains[$key]] = 1;
            } else if ($domains_to_show[$domains[$key]] ==  1) {
                $domains_to_show[$domains[$key]] = Array ($key => $value);
            }
        }
    } else {
        $result_array_temp = $result_array_full;
    }

    foreach ($result_array_temp as $key => $value) {
        $result_array[$key] = $value;
        if ($bound_search_result != 0
            && count($result_array) == $bound_search_result
        ) {
            break;
        }
        if (isset($domains_to_show[$domains[$key]])
            && $domains_to_show[$domains[$key]] != 1
        ) {
            foreach ($domains_to_show as $thisdomain) {
                foreach ($thisdomain as $k => $v) {
                    $result_array[$k] = $v;
                }
            }
        }
    }

    $results = count($result_array);
    $keys = array_keys($result_array);
    $maxweight = $result_array[$keys[0]];

    for ($i = ($start -1)*$per_page; $i <min(
        $results,
        ($start -1)*$per_page + $per_page
    ); $i++) {
        $in[] = $keys[$i];
    }
    if (!is_array($in)) {
        $res['results'] = $results;
        return $res;
    }

    $inlist = implode(",", $in);

    if ($length_of_link_desc == 0) {
        $fulltxt = "fulltxt";
    } else {
        $fulltxt = "substring(fulltxt, 1, $length_of_link_desc)";
    }

    $query1 = "SELECT DISTINCT link_id, url, title, description,  ".$fulltxt
      .", indexdate, size FROM ".$mysql_table_prefix."links WHERE link_id in ("
      .$db->real_escape_string($inlist).")";
    $stmt1 = $db->prepare($query1);
    if ($stmt1) {
        $stmt1->execute() or die("Execution failed: ".$stmt1->error);
        $result = $stmt1->get_result();
        $i = 0;
        $stmt1->close();
    } else {
        trigger_error('Statement failed : '.$db->error, E_USER_ERROR);
    }
    while ($row = $result->fetch_array(MYSQLI_NUM)) {
        $res[$i]['title'] = $row[2];
        $res[$i]['url'] = $row[1];
        if ($row[3] != null && $show_meta_description == 1) {
            $res[$i]['fulltxt'] = $row[3];
        } else {
            $res[$i]['fulltxt'] = $row[4];
        }
        $res[$i]['indexdate'] = $row[5];
        $res[$i]['size'] = $row[6];
        $res[$i]['weight'] = $result_array[$row[0]];
        $url_parts = parse_url($row[1]);
        $domain_for_db = $url_parts['host'];
        $res[$i]['domain'] = $domain_for_db;
        $i++;
    }

    if ($merge_site_results  && $domain_qry == "") {
        sortWithDomains($res);
    } else {
        usort($res, "cmp");
    }
    $res['maxweight'] = $maxweight;
    $res['results'] = $results;

    return $res;
}


/**
 * Get the search results
 *
 * @param string $query      Search qwuery
 * @param int    $start      Results offset
 * @param int    $category   Search category
 * @param string $searchtype Type of search
 * @param array  $results    Array of search results
 * @param int    $domain     Domain being searched
 *
 * @return array $full_result Array of search results
 */
function getSearchResults(
    $query, $start, $category, $searchtype, $results, $domain
) {

    global $sph_messages, $results_per_page, $links_to_next, $show_query_scores;
    global $desc_length, $all_wild, $special_characters_array;

    if ($results != "") {
        $results_per_page = $results;
    }

    if ($searchtype == "phrase") {
        $query=str_replace('"', '', $query);
        $query = "\"".$query."\"";
    } else {
        $query = str_replace($special_characters_array, ' ', $query);
    }

    $starttime = getMicroTime();
    // catch " if only one time entered
    if (mb_substr_count($query, '"', 'utf-8')==1) {
        $query=str_replace('"', '', $query);
        // Now we need to deal with periods and commas which are not
        // specifically a decimal separator
        $query = preg_replace("/^[\.,]/", "", $query);  // beginning of line
        $query = preg_replace("/[\.,]$/", "", $query);  // end of line
        if (preg_match_all("/[^0-9][\.,]/", $query, $matches)) { // no digit before
            foreach ($matches[0] as $pat) {
                $query = str_replace($pat, $pat[0]." ", $query);
            }
        }
        if (preg_match_all("/[\.,][^0-9]/", $query, $matches)) { // no digit after
            foreach ($matches[0] as $pat) {
                $query = str_replace($pat, " ".$pat[1], $query);
            }
        }
    }
    $words = makeBoolList($query);
    $ignorewords = $words['ignore'];

    $full_result['category'] = $category;
    $full_result['type'] = $searchtype;
    $full_result['ignore_words'] = $words['ignore'];

    if ($start==0) {
        $start=1; 
    }
    $result = search(
        $words, $category, $start, $results_per_page, $searchtype, $domain
    );
    $query= stripslashes($query);

    $entitiesQuery = htmlspecialchars($query);
    $full_result['ent_query'] = $entitiesQuery;

    $endtime = getMicroTime() - $starttime;
    $rows = $result['results'];
    $time = round($endtime*100)/100;

    $full_result['time'] = $time;
    
    $did_you_mean = "";

    if (isset($result['did_you_mean'])) {
        $did_you_mean_b=$entitiesQuery;
        $did_you_mean=$entitiesQuery;
        foreach ($result['did_you_mean'] as $key => $val) {
            if ($key != $val) {
                $did_you_mean_b = str_replace(
                    $key, "<b>$val</b>", $did_you_mean_b
                );
                $did_you_mean = str_replace($key, "$val", $did_you_mean);
            }
        }
    }

    $full_result['did_you_mean'] = $did_you_mean;
    $full_result['did_you_mean_b'] = $did_you_mean_b;

    $matchword = $sph_messages["matches"];
    if ($rows == 1) {
        $matchword= $sph_messages["match"];
    }

    // PHP7.2+ distinguishes between null and empty
    if (is_null($result)) {
        $result = [];
    }
    $num_of_results = count($result) - 2;

    $full_result['num_of_results'] = $num_of_results;

    if ($start < 2) {
        saveToLog(addslashes($query), $time, $rows); 
    }
    $from = ($start-1) * $results_per_page+1;
    $to = min(($start)*$results_per_page, $rows);

    $full_result['from'] = $from;
    $full_result['to'] = $to;
    $full_result['total_results'] = $rows;

    if ($rows>0) {
        $maxweight = $result['maxweight'];
        $i = 0;
        while ($i < $num_of_results && $i < $results_per_page) {
            $title = $result[$i]['title'];
            $url = $result[$i]['url'];
            $fulltxt = $result[$i]['fulltxt'];
            $page_size = $result[$i]['size'];
            $domain = $result[$i]['domain'];
            $indexdate = $result[$i]['indexdate'];
            if ($page_size!="") {
                $page_size = number_format($page_size, 1); 
            }
            // If available, enable part of a word highlighting in result report
            if ($all_wild) { 
                $words = makeBoolList($all_wild); 
            }

            $txtlen = mb_strlen($fulltxt, 'utf-8');
            if ($txtlen > $desc_length) {
                $places = array();
                foreach ($words['hilight'] as $word) {
                    $tmp = mb_strtolower($fulltxt ?? '', 'utf-8');
                    $found_in = mb_strpos($tmp, $word, 0, 'utf-8');
                    $sum = -mb_strlen($word, 'utf-8');
                    while (!($found_in =='')) {
                        $pos = $found_in+mb_strlen($word, 'utf-8');
                        $sum += $pos;  //FIX!!
                        $tmp = mb_substr($tmp ?? '', $pos, null, 'utf-8');
                        $places[] = $sum;
                        $found_in = mb_strpos($tmp, $word, 0, 'utf-8');
                    }
                }
                sort($places);
                $x = 0;
                $begin = 0;
                $end = 0;
                foreach ($places as $id => $place) {
                    while ($places[$id + $x] - $place < $desc_length
                      && $x+$id < count($places)
                      && $place < mb_strlen($fulltxt, 'utf-8') -$desc_length) {
                        $x++;
                        $begin = $id;
                        $end = $id + $x;
                    }
                }

                $begin_pos = max(0, $places[$begin] - 30);
                $fulltxt = mb_substr(
                    $fulltxt ?? '', $begin_pos, $desc_length, 'utf-8'
                );

                if ($places[$begin] > 0) {
                    $begin_pos = mb_strpos($fulltxt, " ", 0, 'utf-8');
                }
                $fulltxt = mb_substr(
                    $fulltxt ?? '', $begin_pos, $desc_length, 'utf-8'
                );
                $fulltxt = mb_substr(
                    $fulltxt ?? '', 0, mb_strrpos($fulltxt, " ", 0, 'utf-8'), 'utf-8'
                );
                $fulltxt = $fulltxt;
            }

            $weight = number_format($result[$i]['weight']/$maxweight*100, 2);
            if ($title=='') {
                $title = $sph_messages["Untitled"];
            }
            $regs = Array();

            if (strlen($title) > 80) {
                 $title = mb_substr($title ?? '', 0, 76, 'utf-8')."...";
            }
            foreach ($words['hilight'] as $change) {
                while (preg_match(
                    "/[^\>](".$change.")[^\<]/i", " "
                    .$title." ", $regs
                )) {
                    $title = preg_replace(
                        "/".$regs[1]."/i", "<b>".$regs[1]
                        ."</b>", $title ?? ''
                    );
                }

                while (preg_match(
                    "/[^\>](".$change.")[^\<]/i", " "
                    .$fulltxt." ", $regs
                )) {
                    $fulltxt = preg_replace(
                        "/".$regs[1]."/i", "<b>"
                         .$regs[1]."</b>", $fulltxt ?? ''
                    );
                }
                $url2 = $url." "; // $url2 has a trailing space added
                while (
                    preg_match(
                        "/[^\>](".$change.")[^\<]/i", $url2 ?? '', $regs
                    )) {
                    $url2 = preg_replace(
                        "/".$regs[1]."/i", "<b>"
                        .$regs[1]."</b>", $url2 ?? ''
                    );
                }
                $url2 = trim($url2); // Trailing space removed
            }


            $num = $from + $i;

            $full_result['qry_results'][$i]['num'] =  $num;
            $full_result['qry_results'][$i]['weight'] =  $weight;
            $full_result['qry_results'][$i]['url'] =  $url;
            $full_result['qry_results'][$i]['title'] =  $title;
            $full_result['qry_results'][$i]['fulltxt'] =  $fulltxt;
            $full_result['qry_results'][$i]['url2'] =  $url2;
            $full_result['qry_results'][$i]['page_size'] =  $page_size;
            $full_result['qry_results'][$i]['domain_name'] =  $domain;
            $full_result['qry_results'][$i]['indexdate'] = $indexdate;
            $i++;
        }
    }

    $pages = ceil($rows / $results_per_page);
    $full_result['pages'] = $pages;
    $prev = $start - 1;
    $full_result['prev'] = $prev;
    $next = $start + 1;
    $full_result['next'] = $next;
    $full_result['start'] = $start;
    $full_result['query'] = $entitiesQuery;

    if ($from <= $to) {
        $firstpage = $start - $links_to_next;
        if ($firstpage < 1) {
            $firstpage = 1;
        }
        $lastpage = $start + $links_to_next;
        if ($lastpage > $pages) {
            $lastpage = $pages;
        }
        for ($x=$firstpage; $x<=$lastpage; $x++) {
            $full_result['other_pages'][] = $x;
        }
    }

    return $full_result;

}
